/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 *  WebSphere sMash
 *
 * (C) Copyright IBM Corp. 2007  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */

 /*
  * This file acts as a runtime interface to the RESTdoc tool. It creates REST 
  * API documentation for an application dynamically, so it's always up-to-date.
  *
  * @author Dan Jemiolo (danjemiolo)
  *
  */

import java.util.logging.Logger;

import zero.restdoc.RestDoc;

def createRestDoc()
{
    def restdoc = new RestDoc();
    
    //
    // send tracing output to zero log - this will only show up 
    // if the log level is FINE or lower
    //
    Logger log = Logger.getLogger(restdoc.getClass().getName());
    restdoc.setLog(log);
    restdoc.setVerbose(true);
    
    return restdoc;
}

def error(status, resource)
{
    request.status = status;
    request.view = "error";
    request.error.message = "The application does not have a RESTful resource named '${resource}'.";
    render();
}

/**
 *  
 * @success 200 Returns an HTML page with an index of all the RESTful resources.
 * @format text/html
 *
 */
def onList()
{
    def restdoc = createRestDoc();
    def resourcePaths = [];
    
    config.dependencies[].each() { dependencyName -> 
        def dependencyDir = zget("/config/dependencies/${dependencyName}");
	    def sourceDir = new File(dependencyDir, RestDoc.DEFAULT_INPUT_DIR_PATH);
	    def resourceFiles = restdoc.getSourceFiles(sourceDir);
	    
	    resourceFiles.each() { file -> 
	    	def name = file.getName();
	    	def ext = name.lastIndexOf('.');
	    	resourcePaths << 'restdoc/' + name.substring(0, ext);
	    }
	}
	
    def appSourceDir = new File(config.root[], RestDoc.DEFAULT_INPUT_DIR_PATH);
    restdoc.createIndexFile(request.writer[], appSourceDir, resourcePaths);
}

/**
 *  
 * @success 200 Returns an HTML page with the REST table for the given resource.
 * @error 404 If the given resource does not exist.
 * @format text/html
 *
 */
def onRetrieve()
{
    def restdoc = createRestDoc();
    
    def resourceName = request.params.restdocId[];
    def resourceFile = getVFile(RestDoc.DEFAULT_INPUT_DIR_PATH + '/' + resourceName, ['.groovy', '.php']);
    
    if (resourceFile == null)
        error(404, resourceName);
    
    else
        restdoc.createResourceFile(request.writer[], resourceFile, '../restdoc');
}
